package com.gic.haoban.manage.service.service.out.impl.hm;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.haoban.base.api.common.errCode.HaoBanErrCodeCommon;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.hm.HmPageQueryDTO;
import com.gic.haoban.manage.api.dto.hm.HmPageRelationQueryDTO;
import com.gic.haoban.manage.api.dto.hm.HmPageRelationRespDTO;
import com.gic.haoban.manage.api.dto.hm.HmPageTemplateDTO;
import com.gic.haoban.manage.api.dto.hm.HmPageTemplateRespDTO;
import com.gic.haoban.manage.api.enums.hm.HmPageTemplateStatus;
import com.gic.haoban.manage.api.service.hm.HmPageTemplateApiService;
import com.gic.haoban.manage.service.pojo.bo.hm.HmPageBO;
import com.gic.haoban.manage.service.pojo.bo.hm.HmPageRelationBO;
import com.gic.haoban.manage.service.pojo.qo.HmPageRelationQO;
import com.gic.haoban.manage.service.pojo.qo.HmPageTemplateQO;
import com.gic.haoban.manage.service.service.hm.HmPageRelationService;
import com.gic.haoban.manage.service.service.hm.HmPageService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

/**
 * @Author MUSI
 * @Date 2022/7/3 8:22 PM
 * @Description
 * @Version
 **/
@Component(value = "hmPageTemplateApiServiceImpl")
public class HmPageTemplateApiServiceImpl implements HmPageTemplateApiService {

    private static final Logger log = LoggerFactory.getLogger(HmPageTemplateApiServiceImpl.class);

    private static final String TEMPLATE_CODE_PREFIX = "MB01";
    @Autowired
    private HmPageRelationService pageRelationService;

    @Autowired
    private HmPageService haobanHmPageService;

    @Override
    public ServiceResponse<Long> saveOrUpdateHmPageTemplate(HmPageTemplateDTO pageTemplateDTO) {

        if (Objects.nonNull(pageTemplateDTO.getPageId())) {
            // update
            HmPageBO pageDetail = haobanHmPageService.queryHmPageDetail(pageTemplateDTO.getPageId());
            if (Objects.isNull(pageDetail)) {
                // 数据不存在
                return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_8.getCode() + "", HaoBanErrCodeCommon.ERR_8.getMsg());
            }
            if (!StringUtils.equals(pageDetail.getEnterpriseId(), pageTemplateDTO.getEnterpriseId())) {
                // 无权限
                return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_9.getCode() + "", HaoBanErrCodeCommon.ERR_9.getMsg());
            }
            if (!StringUtils.equals(pageDetail.getWxEnterpriseId(), pageTemplateDTO.getWxEnterpriseId())) {
                // 无权限
                return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_9.getCode() + "", HaoBanErrCodeCommon.ERR_9.getMsg());
            }
        }
        return ServiceResponse.success(haobanHmPageService.saveOrUpdateHmPageTemplate(pageTemplateDTO));
    }

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public ServiceResponse<Boolean> deleteHmPageTemplate(HmPageTemplateDTO pageTemplateDTO) {

        HmPageBO pageBo = haobanHmPageService.queryHmPageDetail(pageTemplateDTO.getPageId());
        if (Objects.isNull(pageBo) || !Objects.equals(HmPageTemplateStatus.ENABLE.getCode(), pageBo.getStatus())) {
            return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_8.getCode() + "", HaoBanErrCodeCommon.ERR_8.getMsg());
        }
        if (this.checkPageIsBeQuoted(pageBo.getPageCode()) > 0) {
            return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_9.getCode() + "", HaoBanErrCodeCommon.ERR_9.getMsg());
        }
        haobanHmPageService.deleteHmPageTemplate(pageTemplateDTO);
        return ServiceResponse.success(Boolean.TRUE);
    }

    @Override
    public ServiceResponse<Page<HmPageTemplateRespDTO>> queryPageTemplateList(HmPageQueryDTO pageQueryDTO) {
        HmPageTemplateQO pageTemplateQo = new HmPageTemplateQO();
        BeanUtils.copyProperties(pageQueryDTO, pageTemplateQo);
        pageTemplateQo.setStatus(HmPageTemplateStatus.ENABLE.getCode());
        Page<HmPageBO> hmPageBoPages = haobanHmPageService.queryPageList(pageTemplateQo);
        Page<HmPageTemplateRespDTO> pageRespVos = PageUtil.changeToCurrentPage(hmPageBoPages, HmPageTemplateRespDTO.class);
        return ServiceResponse.success(pageRespVos);
    }

    /**
     * 校验落地页模板pageTemplateCode 是否被关联
     *
     * @param pageTemplateCode
     * @return true 存在关联
     * false 不存在关联关系
     */
    private Integer checkPageIsBeQuoted(String pageTemplateCode) {
        List<HmPageRelationBO> relations = pageRelationService.queryPageTemplateRelation(pageTemplateCode);
        return 0;
    }

    /**
     * 条件查询落地页关联详情
     *
     * @param relationQueryDTO
     * @return
     */
    @Override
    public ServiceResponse<Page<HmPageRelationRespDTO>> queryPageRelation(HmPageRelationQueryDTO relationQueryDTO) {
        HmPageRelationQO hmPageRelationQo = new HmPageRelationQO();
        hmPageRelationQo.setTemplateSearchText(relationQueryDTO.getSearchText());
        hmPageRelationQo.setEnterpriseId(relationQueryDTO.getEnterpriseId());
        hmPageRelationQo.setWxEnterpriseId(relationQueryDTO.getWxEnterpriseId());
        hmPageRelationQo.setPageNum(relationQueryDTO.getPageNum());
        hmPageRelationQo.setPageSize(relationQueryDTO.getPageSize());
        hmPageRelationQo.setPageCode(relationQueryDTO.getPageId() + "");
        Page<HmPageRelationBO> hmPageRelationPage = pageRelationService.queryPageTemplateRelationWithSearchText(hmPageRelationQo);
        return ServiceResponse.success(PageUtil.changeToCurrentPage(hmPageRelationPage, HmPageRelationRespDTO.class));
    }

    /**
     * 根据页面id 查询
     *
     * @param pageId
     * @return
     */
    @Override
    public ServiceResponse<HmPageTemplateRespDTO> queryPageTemplateDetail(Long pageId) {
        HmPageBO pageBO = haobanHmPageService.queryHmPageDetail(pageId);
        if (pageBO == null) {
            return ServiceResponse.success();
        }
        HmPageTemplateRespDTO hmPageTemplateRespDTO = new HmPageTemplateRespDTO();
        BeanUtils.copyProperties(pageBO, hmPageTemplateRespDTO);
        return ServiceResponse.success(hmPageTemplateRespDTO);
    }
}
